unit u_functions;

{
 TEA   
  
       (public domain)
}


interface

uses sysutils, Classes, StdCtrls;

procedure tea_autostart_shortcut_create (const filename: string; const description: string);
function get_autostart_shortcut_name: string;

function is_ext (filename, ext: string): boolean;
function Bool_To_Str (Value: Boolean): string;
function Str_To_Bool (Value: string): Boolean;
function rnd (min, max: integer): integer;
function GetValue (total, perc: real): real;
function GetPercent (total, value: real): real;
function MakePrefixedFileName (_path, prefix: string): string;
function is_office (filename: string): boolean;
function make_ver_filename (filename: string): string;
function is_c_like (filename: string): boolean;
function is_pas (filename: string): boolean;
function is_image (filename: string): boolean;
function AnalyzeDocWeight (const s, filename: string): string;


implementation

uses windows, JclStrings, JclFileUtils, u_strings,
     ShlObj, JclShell, JclSysInfo;

function is_ext (filename, ext:string): boolean;
begin
     result := (AnsiLowerCase (extractFileExt (filename)) =
                AnsiLowerCase (ext));
end;


function Bool_To_Str (Value: Boolean): string;
begin
     if Value then
        Result := '1'
     else
         Result := '0';
end;


function Str_To_Bool (Value: string): Boolean;
begin
     if Value='1' then
        Result := true
     else
         Result := false;
end;


function rnd (min, max: integer): integer;
begin
     result := (min) + Random ((max - min) + 1)
end;


procedure FreeFree (var p: TObject);
begin
     try
        if p <> nil then
          p.FRee;
     finally
            p := nil;
     end
end;


function GetValue (total, perc: real): real;
begin
     result := (total / 100) * perc
end;


function GetPercent (total, value: real): real;
begin
     result := (value / total) * 100
end;


function MakePrefixedFileName (_path, prefix: string): string;
var
   path, name: string;
begin
     path := ExtractFilePath (_path);
     name := ExtractFileName (_path);
     result := path + prefix + name;
end;


function make_ver_filename (filename: string): string;
var
   drive, path, name, ext, postfix: string;
begin
     PathExtractElements (filename, Drive, Path, name, Ext);

     postfix := '_' + DateToStr (now);
     postfix := postfix + '_' + TimeToStr (now);

     CharReplace (postfix, ':', '_');
     CharReplace (postfix, '.', '_');

     result := drive + PathSeparator + path + PathSeparator +
               name + postfix + ext;
end;


function is_office (filename: string): boolean;
begin
     if Is_Ext (filename, '.odt') or
        Is_Ext (filename, '.kwd') or
        Is_Ext (filename, '.sxw') then
        result:= true
     else
         result := false;
end;


function is_c_like (filename: string): boolean;
begin
     result := false;
     if is_ext (filename, '.c') or is_ext (filename, '.cpp')
        or is_ext (filename, '.h') then
     result := true;
end;


function is_pas (filename: string): boolean;
begin
     result := false;
     if is_ext (filename, '.pas') or is_ext (filename, '.dpr')
        or is_ext (filename, '.pp') then
     result := true;
end;


function is_image (filename: string): boolean;
begin
     result := false;
     if is_ext (filename, '.jpg') or is_ext (filename, '.jpeg')
        or is_ext (filename, '.gif') or is_ext (filename, '.png')
        or is_ext (filename, '.ps')
        or is_ext (filename, '.tiff') or is_ext (filename, '.bmp')
        or is_ext (filename, '.wbmp') or is_ext (filename, '.eps') then
        result := true;
end;


function AnalyzeDocWeight (const s, filename: string): string;
var
   i, j: integer;
   l: TStringList;
   t, ff: string;
   srclen, slen: integer;
   totalsize, fsize: integer;

begin
     //   SRC
     l := TStringList.Create;

     l.Duplicates := dupIgnore;
     l.Sorted := true;

     i := 1;
     srclen := length ('src');
     slen := length (s);

     while i <> 0 do
           begin
                i := StrFind ('src', s, i + 1);
                t := Copy (s, i + srclen, slen - (i + srclen));
                j := Pos ('"', t);
                t := Copy (t, j + 1, length (t) - j);
                t := StrBefore ('"', t);

                ff := build_path (ExtractFilePath (filename), t);
                if FileExists (ff) then
                   l.add (ff);
           end;

     l.Sorted := false;

     for i:=0 to l.count-1 do
         begin
              fsize := FileGetSize (l[i]);
              totalsize := totalsize + fsize;
              l[i] := l[i] + ' ' + IntToStr (fsize);
         end;

     totalsize := totalsize + FileGetSize (filename);

     l.Add (filename + ' ' + IntToStr (FileGetSize (filename)));
     l.Add ('bytes total: ' + IntToStr (totalsize));

     result := l.text;
     l.Free;
end;


procedure tea_autostart_shortcut_create (const filename: string; const description: string);
var ShellLink: TShellLink;
begin
     with ShellLink do
          begin
               Arguments := '';
               ShowCmd := sw_show;
               WorkingDirectory := ExtractFilePath (filename);
               IdList := nil;
               Target := filename;
               Description:= description;
//                IconLocation:=ExtractFilePath (filename) + '\tea_app.ico';//paramstr(0);
               IconIndex := 0;
               HotKey := 0;
           end;

     ShellLinkCreateSystem (ShellLink, CSIDL_STARTUP, 'TEA.lnk');
     ShellLinkFree (ShellLink);
end;

function get_autostart_shortcut_name: string;
begin
     result := GetStartupFolder + '\' + 'TEA.lnk';
end;


end.
