unit u_sci_editor;

{
 TEA   
  
       (public domain)
}


interface

uses u_virtual_editor,
     u_fm_sci_child,
     forms,
     SciFileExtensionsManager;

type

TSciEditor = class (TVirtualEditor)

form: Tfm_sci_child;
mapper: TExtensionMapper;

constructor create (ownr: TEditors);
destructor destroy; override;

procedure toggle_word_wrap; override;
procedure set_word_wrap (value: boolean); override;


procedure file_load (filename,encoding: string; position: integer); override;
procedure file_save (filename: string); override;
procedure file_save_silent (filename: string); override;
procedure file_save_format (format: string; filename: string); override;


procedure set_enc (const e: string); override;
procedure set_modified (value: boolean); override;

function get_selection_w: WideString; override;
procedure set_selection_w (const s: WideString); override;
function get_text_w: WideString; override;
procedure set_text_w (const s: WideString); override;
procedure ins_at_cursor_w (s: WideString); override;

procedure update_hl; override;

procedure goto_matching_brace; override;
procedure print; override;


procedure redo; override;
procedure undo; override;
procedure cut; override;
procedure copy_to_clipboard; override;
procedure paste; override;
procedure select_all; override;
procedure delete_current_line; override;
procedure indent; override;
procedure unindent; override;
procedure goto_line (n: string); override;
procedure ins_at_cursor (s: string); override;
function get_selection: string; override;
procedure set_selection (const s: string); override;
function get_text: string; override;
procedure set_text (const s: string); override;
procedure update_status; override;
procedure update_settings; override;
procedure ins_image (s: string; markup_mode: integer); override;
procedure set_focus; override;
function get_file_triplex: string;override;
function get_filename_at_cursor: string; override;
function is_selected: boolean; override;
procedure find_from_cursor; override;
procedure replace_all (const find_what, replace_on: string); override;
procedure set_sel_start (value: integer); override;


end;


implementation


uses SysUtils, u_globals, Classes, u_form_main,
     u_strings, JclStrings, jclFileUtils, u_charset,
     u_settings, SynEdit, Graphics, imgsize,
     u_functions, scilexermemo, scilexer, scilexermod,
     dialogs, windows, controls, sciPrint;


procedure TSciEditor.file_load (filename, encoding: string; position: integer);
var
   s, x, enc: string;
begin
     if not FileExists (filename) then
        exit;

     x := FileToString (filename);

     line_ending := get_line_ending (x);

     enc := encoding;

     if enc = c_enc_utf8 then
        form.Scintilla1.CodePage := cpUTF8
     else
         form.Scintilla1.CodePage := cpAnsi;

     if cur_encoding = c_enc_auto then
        enc := get_enc (x);

     if enc = c_enc_utf8 then
        s := x;//UTF8ToANSI (x);

     if enc = c_enc_ansi then
        s := x;

     if enc = c_enc_koi8r then
        s := KOI2ANSI (x);

     if enc = c_enc_koi8u then
        s := KOI8U2ANSI (x);

     if enc = c_enc_oem then
        s := IBM2ANSI (x);

     form.Scintilla1.Lines.Text := s;
     form.Scintilla1.SelStart := position;
     form.Scintilla1.SelLength := 0;

     file_name := filename;
     character_set := enc;
     form.Caption := filename;
     tab.Caption := ExtractFileName (filename);
     saved := true;
     log (filename + '  .');

     form.Scintilla1.LanguageManager.SelectedLanguage := mapper.DetectLanguage (ExtractFileExt (file_name));
     form.Scintilla1.Modified := false;

     update_status;
end;


procedure TSciEditor.file_save (filename: string);
var
   buf: string;
begin
     if readonly then
        begin
             log ('   -    .', clred, true);
             exit;
        end;

     if character_set = c_enc_utf8 then
        buf := form.Scintilla1.Lines.text
     else
         if character_set = c_enc_ansi then
            buf := form.Scintilla1.Lines.text
         else
             if character_set = c_enc_koi8r then
                buf := ansi2KOI (form.Scintilla1.Lines.text)
         else
             if character_set = c_enc_oem then
                buf := ANSI2IBM (form.Scintilla1.Lines.text)
         else
             if character_set = c_enc_koi8u then
                buf := ANSI2KOI8U (form.Scintilla1.Lines.text);

     if line_ending_changed then
        begin
             if line_ending = 'lf' then
                buf := AdjustLineBreaks(buf, tlbsLF)
             else
                 buf := AdjustLineBreaks(buf, tlbsCRLF);
        end;

     StringToFile (filename, buf);

     file_name := filename;
     form.Caption := filename;
     tab.Caption := ExtractFileName (filename);
     saved := TRUE;
     form.Scintilla1.Modified := false;
     log (filename + '  .');

     update_status;
end;


constructor TSciEditor.create (ownr: TEditors);
begin
     inherited create (ownr);

     form := Tfm_sci_child.Create (ownr.main_form);
     metaform := form;
     file_name := 'noname';
     form.Caption := 'file_name';
     form.show;
     e_owner.page_control.ActivePage := tab;
     form.tab := tab;

     mapper := ExtMapper;
     mapper.FillExtensionMap (form.Scintilla1.LanguageManager);

     if settings.def_utf8 then
        set_enc (c_enc_utf8);

     form.Scintilla1.Modified := false;

     update_settings;
     update_status;
end;


destructor TSciEditor.destroy;
var
   s: string;
begin
     tab.Free;

     if file_name = settings.crapbook_file then
       StringToFile (settings.crapbook_file, form.Scintilla1.Lines.text);

     if (FileExists (file_name)) and (file_name <> settings.crapbook_file) then
        begin
             if settings.confirm_mod_save and form.Scintilla1.modified then
               begin
                    s := ' '+ ExtractFileName (file_name) + '  .' +
                    #13#10 + ' ?';

                    if application.MessageBox (PChar (s), '!', MB_YESNO) = ID_YES then
                      file_save_silent (file_name);
               end;

             sl_recent.Insert (0, get_file_triplex);
             if sl_recent.Count > settings.recent_items_limit + 1 then
                sl_recent.Delete (sl_recent.Count - 1);

             e_owner.update_recent;
        end;

     e_owner.editors.Remove (self);
     inherited;
end;


procedure TSciEditor.ins_at_cursor (s: string);
begin
     form.Scintilla1.SelText := s;
end;


function TSciEditor.get_selection: string;
begin
     if form.Scintilla1.SelLength <> 0 then
        result := form.Scintilla1.SelText
     else
         result := form.Scintilla1.Lines.Text;
end;


procedure TSciEditor.set_selection (const s: string);
begin
     if s = '' then
        exit;

     if form.Scintilla1.SelLength <> 0 then
        form.Scintilla1.SelText := s
     else
         form.Scintilla1.lines.Text := s;
end;


function TSciEditor.get_text: string;
begin
     result := form.Scintilla1.lines.Text;
end;


procedure TSciEditor.set_text (const s: string);
begin
     if s <> '' then
        form.Scintilla1.lines.Text := s;
end;


procedure TSciEditor.update_status;
begin
     e_owner.status_bar.Panels[c_cb_docstatus].Text := line_ending + ' ' + character_set;
end;


function TSciEditor.get_file_triplex: string;
begin
     if not FileExists (file_name)
        then
            result := ''
        else
            result := file_name + ',' + character_set + ',' + inttostr (form.Scintilla1.SelStart);
end;


function TSciEditor.get_filename_at_cursor: string;
var
   start, ending, current_pos: integer;
   x: integer;
begin
     if is_selected then
        begin
            result := form.Scintilla1.SelText;
            exit;
        end;

     current_pos := form.Scintilla1.SelStart;
     x := length (form.Scintilla1.Lines.text);

     for ending := current_pos to x do
        if (form.Scintilla1.Lines.text [ending] = '''') or
           (form.Scintilla1.Lines.text [ending] = '"') then
            break;

     for start := current_pos downto 1 do
        if (form.Scintilla1.Lines.text [start] = '''') or
           (form.Scintilla1.Lines.text [start] = '"') then
           break;

    result := Copy (form.Scintilla1.Lines.text, start + 1, ending - start - 1);
end;


procedure TSciEditor.file_save_silent (filename: string);
var
   buf: string;
begin
     if character_set = c_enc_utf8 then
        buf := form.Scintilla1.Lines.text
     else
         if character_set = c_enc_ansi
            then
                buf := form.Scintilla1.Lines.text
         else
             if character_set = c_enc_koi8r
                then
                    buf := ansi2KOI (form.Scintilla1.Lines.text)
         else
             if character_set = c_enc_oem
                then
                    buf := ANSI2IBM (form.Scintilla1.Lines.text)
         else
             if character_set = c_enc_koi8u
                then
                    buf := ANSI2KOI8U (form.Scintilla1.Lines.text);

     if line_ending_changed then
        begin
             if line_ending = 'lf' then
               buf := AdjustLineBreaks (buf, tlbsLF)
             else
                 buf := AdjustLineBreaks (buf, tlbsCRLF);
        end;

     StringToFile (filename, buf);
     log (filename + '  .');
end;


function TSciEditor.is_selected: boolean;
begin
     result := form.Scintilla1.SelLength <> 0;
end;


procedure TSciEditor.update_settings;
begin
     form.SciAutoComplete1.IgnoreCase := settings.auto_comp_ignore_case;

     with form.Scintilla1 do
          begin
               color := settings.bg_color;

               BraceHilite := settings.braces_hl;

               form.SciAutoComplete1.Disabled := not settings.auto_comp;

               caret.ForeColor := settings.caret_color;

               autoclosebraces := settings.autoclose_braces;
               autoclosequotes := settings.autoclose_quotes;

               if settings.show_wraps then
                  WordWrapVisualFlags := [sciWWEnd]
               else
                   WordWrapVisualFlags := [];

               if settings.show_wspaces then
                  divoptions.viewWSpace := sciWsVisibleAlways
               else
                   divoptions.viewWSpace := sciWsInvisible;

               if settings.folding then
                  begin
                       folding := folding + [foldFold];
                       FoldAll;
                  end
               else
                  folding := folding - [foldFold];



               font.Name := settings.font_name;
               font.Color := settings.font_color;
               font.Size := settings.font_size;

               if settings.ShowLineNumbers then
                  gutter0.Width := 64;

               if settings.eoautoindent then
                   Indentation := Indentation + [KeepIndent]
               else
                   Indentation := Indentation - [KeepIndent];


               caret.LineVisible := settings.hl_under_cursor;
               caret.LineBackColor := settings.active_line_color;

                {
               if settings.eoScrollPastEol then
                  options := options + [eoScrollPastEol]
               else
                  options := options - [eoScrollPastEol];
                 }
                 {
               if settings.eoShowSpecialChars then
                  options := options + [eoShowSpecialChars]
               else
                  options := options - [eoShowSpecialChars];
                  }

               if settings.eoTabIndent then
                  indentation := indentation + [TabIndents]
               else
                   indentation := indentation - [TabIndents];

               if settings.right_edge_visible then
                  EdgeMode := sciEdgeLine
               else
                   EdgeMode := sciEdgeNone;

               EdgeColumn := settings.right_edge;
               EdgeColor := settings.right_edge_color;

               colors.SelBack := settings.selected_color_bg;
               colors.SelFore := settings.selected_color_fg;

               tabWidth := settings.tab_width;

               if settings.word_wrap then
                  wordWrap := sciWrap
               else
                   WordWrap := sciNoWrap;

           end;
end;


procedure TSciEditor.ins_image (s: string; markup_mode: integer);
var
   h, w: word;
   filename: string;
   t: string;
begin
     if not is_image (s) then
        exit;

     h := 0;
     w := 0;

     if is_ext (s, '.jpg') or is_ext (s, '.jpeg') then
        GetJPGSize (s, w, h);

     if is_ext (s, '.png') then
        GetPNGSize (s, w, h);

     if is_ext (s, '.gif') then
        GetGIFSize (s, w, h);

     filename := PathGetRelativePath (ExtractFilePath (file_name), s);

     StrReplace (filename, '\', '/', [rfReplaceAll]);

     if markup_mode = c_mm_html then
        t := '<img src="' + filename + '" border="0" alt="" width="' + IntToStr (w) + '" height="' + IntToStr (h) + '">';

     if markup_mode = c_mm_xhtml then
        t := '<img src="' + filename + '" border="0" alt="" width="' + IntToStr (w) + '" height="' + IntToStr (h) + '" />';

     if markup_mode = c_mm_docbook then
        t := '<mediaobject><imageobject><imagedata fileref="' + filename + '"/></imageobject></mediaobject>';

     if markup_mode = c_mm_tex then
        t := '\includegraphics{' + filename + '}';

     ins_at_cursor (t);
end;



procedure TSciEditor.cut;

begin

     form.Scintilla1.Cut;

end;


procedure TSciEditor.copy_to_clipboard;
begin
     form.Scintilla1.Copy;

end;



procedure TSciEditor.paste;
begin

     form.Scintilla1.Paste;

end;



procedure TSciEditor.select_all;

begin

     form.Scintilla1.SelectAll;

end;



procedure TSciEditor.delete_current_line;

begin

     form.Scintilla1.Lines.delete (form.Scintilla1.GetCurrentLineNumber);

end;


procedure TSciEditor.indent;
begin
     form.Scintilla1.tab;
end;


procedure TSciEditor.unindent;
begin
     form.Scintilla1.BackTab;

end;


procedure TSciEditor.goto_line (n: string);
begin
     if not StrIsDigit (n) then
        exit;

     form.Scintilla1.GotoLine (StrToInt (n));
     set_focus;
     form.Scintilla1.Modified := false;
end;


procedure TSciEditor.set_focus;
begin
     form.ActiveControl := form.Scintilla1;
end;


procedure TSciEditor.find_from_cursor;
var
   pos: integer;
begin
     if text_to_find = '' then
        exit;

     pos := StrFind (text_to_find, form.Scintilla1.lines.Text, form.Scintilla1.selstart + 2);

     if pos <> 0 then
        begin
             form.Scintilla1.SelStart := pos - 1;
             form.Scintilla1.SelLength := length (text_to_find);
        end;

     form.Scintilla1.Modified := false;
end;


procedure TSciEditor.redo;
begin
     form.Scintilla1.Redo;
end;


procedure TSciEditor.undo;
begin
     form.Scintilla1.Undo;
end;


procedure TSciEditor.set_enc (const e: string);
begin
     if (character_set = c_enc_utf8) and (e <> c_enc_utf8) then
        form.Scintilla1.Lines.Text := Utf8ToAnsi (form.Scintilla1.Lines.text);

     character_set := e;
     if character_set = c_enc_utf8 then
        begin
             if form.Scintilla1.CodePage <> cpUTF8 then
                form.Scintilla1.Lines.Text := AnsiToUtf8 (form.Scintilla1.Lines.text);

             form.Scintilla1.CodePage := cpUTF8;
        end
     else
         form.Scintilla1.CodePage := cpAnsi;

     update_status;
end;


function TSciEditor.get_selection_w: WideString;
begin
     if is_selected then
        result := UTF8Decode (form.Scintilla1.SelText)
     else
         result := UTF8Decode (form.Scintilla1.Lines.text);
end;


procedure TSciEditor.set_selection_w (const s: WideString);
begin
     if is_selected then
        form.Scintilla1.SelText := UTF8Encode (s)
     else
        form.Scintilla1.Lines.Text := UTF8Encode (s);
end;


function TSciEditor.get_text_w: WideString;
begin
     result := UTF8Decode (form.Scintilla1.Lines.text);
end;


procedure TSciEditor.set_text_w (const s: WideString);
begin
     form.Scintilla1.Lines.Text := UTF8Encode (s);
end;


procedure TSciEditor.ins_at_cursor_w (s: WideString);
begin
     form.Scintilla1.SelText := UTF8Encode (s)
end;


procedure TSciEditor.replace_all (const find_what, replace_on: string);
begin
//to do
end;


procedure TSciEditor.update_hl;
begin
     form.Scintilla1.LanguageManager.SelectedLanguage := mapper.DetectLanguage (ExtractFileExt (file_name));
end;


procedure TSciEditor.set_modified (value: boolean);
begin
     form.Scintilla1.Modified := value;
end;


procedure TSciEditor.set_sel_start (value: integer);
begin
     form.Scintilla1.SelStart := value;
end;


procedure TSciEditor.toggle_word_wrap;
begin
     if form.Scintilla1.WordWrap = sciNoWrap then
        form.Scintilla1.WordWrap := sciWrap
     else
         form.Scintilla1.WordWrap := sciNoWrap;
end;


procedure TSciEditor.set_word_wrap (value: boolean);
begin
     if value then
        form.Scintilla1.WordWrap := sciWrap
     else
         form.Scintilla1.WordWrap := sciNoWrap;
end;


procedure TSciEditor.goto_matching_brace;
begin
     form.Scintilla1.GoMatchingBrace (true, true);
end;


procedure TSciEditor.print;
begin
     fm_main.SciPrinter1.Editor := form.Scintilla1;
     fm_main.SciPrinter1.WordWrap := form.Scintilla1.WordWrap;
     fm_main.SciPrinter1.Title := file_name;
     fm_main.SciPrinter1.FromPage := settings.prn_from_page;
     fm_main.SciPrinter1.ToPage := settings.prn_to_page;
     fm_main.SciPrinter1.MarginTop := settings.prn_margin_top;
     fm_main.SciPrinter1.MarginBottom := settings.prn_margin_bottom;
     fm_main.SciPrinter1.MarginLeft := settings.prn_margin_left;
     fm_main.SciPrinter1.MarginRight := settings.prn_margin_right;
     fm_main.SciPrinter1.PrintPageNumber := settings.prn_print_page_nums;

     if settings.prn_print_range then
        fm_main.SciPrinter1.PrintRange := prPageNums
     else
         fm_main.SciPrinter1.PrintRange := prAllPages;

     if settings.prn_blackwhite then
        fm_main.SciPrinter1.ColorMode := sccmBlackOnWhite
     else
         fm_main.SciPrinter1.ColorMode := sccmNormal;

     fm_main.SciPrinter1.Print;
end;


procedure TSciEditor.file_save_format (format: string; filename: string);
var
   fs: TMemoryStream;
begin
     if format = 'HTML' then
        begin
             fs := TMemoryStream.Create;
             form.Scintilla1.ExportToHTML (fs, ExtractFileName (file_name));
             fs.SaveToFile (filename);
             fs.Free;
        end;
end;


end.
