unit u_virtual_editor;

{
 TEA   
  
       (public domain)
}


interface

uses Classes, Forms, Contnrs, ComCtrls;


type

TPlainProc = procedure of object;

TEditors = class

main_form: TForm;
page_control: TPageControl;
status_bar: TStatusBar;

editors: TList;


update_recent: TPlainProc;
save_recent: TPlainProc;


constructor Create;
destructor destroy;override;
function is_already_opened (filename: string): pointer;
procedure update_settings;



end;


TVirtualEditor = class

readonly: boolean;
e_owner: TEditors;
metaform: TForm;
saved: boolean;
tab: TTabSheet;

text_to_find: string;

line_ending: string;

line_ending_changed: boolean;


tab_index: integer;
character_set: string;
file_name: string;

found: integer;

constructor Create (ownr: TEditors);

procedure do_show (Sender: TObject); virtual;
procedure do_enter (Sender: TObject); virtual;

procedure set_enc (const e: string); virtual; abstract;


function is_selected: boolean; virtual; abstract;
function is_utf8: boolean; 


function get_selection: string; virtual; abstract;
procedure set_selection (const s: string); virtual; abstract;
function get_text: string; virtual; abstract;
procedure set_text (const s: string); virtual; abstract;


function get_selection_w: WideString; virtual; abstract;
procedure set_selection_w (const s: WideString); virtual; abstract;
function get_text_w: WideString; virtual; abstract;
procedure set_text_w (const s: WideString); virtual; abstract;
procedure ins_at_cursor_w (s: WideString); virtual; abstract;

procedure find_from_cursor; virtual; abstract;

procedure goto_line (n: string); virtual; abstract;
procedure set_focus; virtual; abstract;


procedure file_load (filename, encoding: string; position: integer); virtual; abstract;
procedure file_save (filename: string); virtual; abstract;
procedure file_save_silent (filename: string); virtual; abstract;
procedure file_save_format (format: string; filename: string); virtual; abstract;


procedure update_settings; virtual; abstract;


procedure redo; virtual; abstract;
procedure undo; virtual; abstract;


procedure cut; virtual; abstract;
procedure copy_to_clipboard; virtual; abstract;
procedure paste; virtual; abstract;

procedure select_all; virtual; abstract;
procedure delete_current_line; virtual; abstract;


procedure indent; virtual; abstract;
procedure unindent; virtual; abstract;


procedure print; virtual; abstract;


procedure update_status; virtual; abstract;
procedure ins_at_cursor (s: string); virtual; abstract;

procedure update_hl; virtual; abstract;
procedure set_modified (value: boolean); virtual; abstract;
procedure set_sel_start (value: integer); virtual; abstract;

procedure toggle_word_wrap; virtual; abstract;
procedure set_word_wrap (value: boolean); virtual; abstract;


procedure goto_matching_brace; virtual; abstract;


procedure ins_image (s: string; markup_mode: integer); virtual; abstract;


function get_file_triplex: string; virtual; abstract;
function get_filename_at_cursor: string; virtual; abstract;

procedure replace_all (const find_what, replace_on: string); virtual; abstract;


end;


implementation

uses u_globals;


constructor TVirtualEditor.Create (ownr: TEditors);
var
   ts: TTabSheet;
begin
     found := 0;
     readonly := false;
     e_owner := ownr;
     e_owner.editors.Add (self);
     saved := false;
     character_set := c_enc_ansi;
     ts:=TTabSheet.Create(ownr.page_control);
     ts.PageControl := e_owner.page_control;
     ts.Caption:= ' '; //cUntitled+IntToStr(ct.iuntitled);
//     ts.OnShow := do_show;
   //  ts.OnEnter := do_enter;

     text_to_find := '';
     tab := ts;
     line_ending := 'crlf';
     line_ending_changed := false;
end;

constructor TEditors.Create;
begin
      inherited;
      editors := TList.Create;

end;

destructor TEditors.destroy;
var i, c: integer;
begin
     c := editors.Count - 1;
     if (c <> -1) then
     for i := 0 to c do
         TVirtualEditor(editors[0]).free;

     editors.Free;
     inherited;
end;


procedure TEditors.update_settings;
var i, c: integer;
begin
     c := editors.Count - 1;
     if (c <> -1) then
     for i := 0 to c do
         TVirtualEditor(editors[i]).update_settings;
end;



function TEditors.is_already_opened (filename: string): pointer;
var
   i: integer;

begin
     result := nil;
     if editors.Count = 0 then
        exit;

     for i := 0 to editors.Count - 1 do
         if TVirtualEditor(editors[i]).file_name = filename then
            begin
                 result := editors[i];
                 exit;
            end;
end;


procedure TVirtualEditor.do_show (Sender: TObject);
begin
     metaform.SetFocus;

end;

procedure TVirtualEditor.do_enter (Sender: TObject);
begin
     metaform.SetFocus;
end;


function TVirtualEditor.is_utf8;
begin
     result := character_set = c_enc_utf8;
end;





end.
